"""
腾讯云 AS 服务主模块
"""
from asyncio.log import logger
from typing import Any
from mcp.server.models import InitializationOptions
import mcp.types as types
from mcp.server import NotificationOptions, Server
import mcp.server.stdio
from . import tool_as

server = Server("as")

@server.list_tools()
async def handle_list_tools() -> list[types.Tool]:
    """处理工具列表请求"""
    return [
        types.Tool(
            name="CreateAutoScalingGroup",
            description="创建伸缩组",
            inputSchema={
                "type": "object",
                "properties": {
                    "Region": {
                        "type": "string",
                        "description": "地域，如 ap-guangzhou",
                    },
                    "AutoScalingGroupName": {
                        "type": "string",
                        "description": "伸缩组名称",
                    },
                    "LaunchConfigurationId": {
                        "type": "string",
                        "description": "启动配置ID",
                    },
                    "MaxSize": {
                        "type": "integer",
                        "description": "最大实例数",
                    },
                    "MinSize": {
                        "type": "integer",
                        "description": "最小实例数",
                    },
                    "VpcId": {
                        "type": "string",
                        "description": "VPC ID",
                    },
                    "SubnetIds": {
                        "type": "array",
                        "items": {"type": "string"},
                        "description": "子网ID列表",
                    },
                    "DesiredCapacity": {
                        "type": "integer",
                        "description": "期望实例数",
                    }
                },
                "required": ["Region", "AutoScalingGroupName", "LaunchConfigurationId", "MaxSize", "MinSize", "VpcId", "SubnetIds"],
            },
        ),
        types.Tool(
            name="DescribeAutoScalingGroups",
            description="查询伸缩组",
            inputSchema={
                "type": "object",
                "properties": {
                    "Region": {
                        "type": "string",
                        "description": "地域，如 ap-guangzhou",
                    },
                    "AutoScalingGroupIds": {
                        "type": "array",
                        "items": {"type": "string"},
                        "description": "伸缩组ID列表",
                    },
                    "Filters": {
                        "type": "array",
                        "items": {
                            "type": "object",
                            "properties": {
                                "Name": {"type": "string"},
                                "Values": {
                                    "type": "array",
                                    "items": {"type": "string"}
                                }
                            }
                        },
                        "description": "过滤条件",
                    },
                    "Limit": {
                        "type": "integer",
                        "description": "返回数量",
                    },
                    "Offset": {
                        "type": "integer",
                        "description": "偏移量",
                    }
                },
                "required": ["Region"],
            },
        ),
        types.Tool(
            name="ModifyAutoScalingGroup",
            description="修改伸缩组",
            inputSchema={
                "type": "object",
                "properties": {
                    "Region": {
                        "type": "string",
                        "description": "地域，如 ap-guangzhou",
                    },
                    "AutoScalingGroupId": {
                        "type": "string",
                        "description": "伸缩组ID",
                    },
                    "MaxSize": {
                        "type": "integer",
                        "description": "最大实例数",
                    },
                    "MinSize": {
                        "type": "integer",
                        "description": "最小实例数",
                    },
                    "DesiredCapacity": {
                        "type": "integer",
                        "description": "期望实例数",
                    }
                },
                "required": ["Region", "AutoScalingGroupId"],
            },
        ),
        types.Tool(
            name="EnableAutoScalingGroup",
            description="启用伸缩组",
            inputSchema={
                "type": "object",
                "properties": {
                    "Region": {
                        "type": "string",
                        "description": "地域，如 ap-guangzhou",
                    },
                    "AutoScalingGroupId": {
                        "type": "string",
                        "description": "伸缩组ID",
                    }
                },
                "required": ["Region", "AutoScalingGroupId"],
            },
        ),
        types.Tool(
            name="DisableAutoScalingGroup",
            description="停用伸缩组",
            inputSchema={
                "type": "object",
                "properties": {
                    "Region": {
                        "type": "string",
                        "description": "地域，如 ap-guangzhou",
                    },
                    "AutoScalingGroupId": {
                        "type": "string",
                        "description": "伸缩组ID",
                    }
                },
                "required": ["Region", "AutoScalingGroupId"],
            },
        ),
        types.Tool(
            name="ExecuteScalingPolicy",
            description="执行伸缩策略",
            inputSchema={
                "type": "object",
                "properties": {
                    "Region": {
                        "type": "string",
                        "description": "地域，如 ap-guangzhou",
                    },
                    "AutoScalingGroupId": {
                        "type": "string",
                        "description": "伸缩组ID",
                    },
                    "Operation": {
                        "type": "string",
                        "description": "操作类型",
                        "enum": ["SCALE_OUT", "SCALE_IN"],
                    },
                    "AdjustmentType": {
                        "type": "string",
                        "description": "调整类型",
                        "enum": ["CHANGE_IN_CAPACITY", "EXACT_CAPACITY", "PERCENT_CHANGE_IN_CAPACITY"],
                    },
                    "AdjustmentValue": {
                        "type": "integer",
                        "description": "调整值",
                    }
                },
                "required": ["Region", "AutoScalingGroupId", "Operation"],
            },
        ),
        types.Tool(
            name="ModifyDesiredCapacity",
            description="修改期望实例数",
            inputSchema={
                "type": "object",
                "properties": {
                    "Region": {
                        "type": "string",
                        "description": "地域，如 ap-guangzhou",
                    },
                    "AutoScalingGroupId": {
                        "type": "string",
                        "description": "伸缩组ID",
                    },
                    "DesiredCapacity": {
                        "type": "integer",
                        "description": "期望实例数",
                    }
                },
                "required": ["Region", "AutoScalingGroupId", "DesiredCapacity"],
            },
        ),
    ]

@server.call_tool()
async def handle_call_tool(
    name: str, arguments: dict[str, Any] | None
) -> list[types.TextContent | types.ImageContent | types.EmbeddedResource]:
    """处理工具调用请求"""
    try:
        region = arguments.get("Region")
        
        if name == "CreateAutoScalingGroup":
            result = tool_as.create_auto_scaling_group(region, arguments)
        elif name == "DescribeAutoScalingGroups":
            result = tool_as.describe_auto_scaling_groups(
                region=region,
                auto_scaling_group_ids=arguments.get("AutoScalingGroupIds"),
                filters=arguments.get("Filters"),
                limit=arguments.get("Limit"),
                offset=arguments.get("Offset")
            )
        elif name == "ModifyAutoScalingGroup":
            result = tool_as.modify_auto_scaling_group(
                region=region,
                auto_scaling_group_id=arguments.get("AutoScalingGroupId"),
                params=arguments
            )
        elif name == "EnableAutoScalingGroup":
            result = tool_as.enable_auto_scaling_group(
                region=region,
                auto_scaling_group_id=arguments.get("AutoScalingGroupId")
            )
        elif name == "DisableAutoScalingGroup":
            result = tool_as.disable_auto_scaling_group(
                region=region,
                auto_scaling_group_id=arguments.get("AutoScalingGroupId")
            )
        elif name == "ExecuteScalingPolicy":
            result = tool_as.execute_scaling_policy(
                region=region,
                auto_scaling_group_id=arguments.get("AutoScalingGroupId"),
                operation=arguments.get("Operation"),
                adjustment_type=arguments.get("AdjustmentType"),
                adjustment_value=arguments.get("AdjustmentValue")
            )
        elif name == "ModifyDesiredCapacity":
            result = tool_as.modify_desired_capacity(
                region=region,
                auto_scaling_group_id=arguments.get("AutoScalingGroupId"),
                desired_capacity=arguments.get("DesiredCapacity")
            )
        else:
            raise ValueError(f"未知的工具: {name}")
            
        return [types.TextContent(type="text", text=str(result))]
    except Exception as e:
        return [types.TextContent(type="text", text=f"错误: {str(e)}")]

async def serve():
    """启动服务"""
    async with mcp.server.stdio.stdio_server() as (read_stream, write_stream):
        logger.info("Server running with stdio transport")
        await server.run(
            read_stream,
            write_stream,
            InitializationOptions(
                server_name="as",
                server_version="0.1.0",
                capabilities=server.get_capabilities(
                    notification_options=NotificationOptions(),
                    experimental_capabilities={},
                ),
            ),
        )



"""
腾讯云 AS 客户端模块
"""
import os
from tencentcloud.common import credential
from tencentcloud.common.profile.client_profile import ClientProfile
from tencentcloud.common.profile.http_profile import HttpProfile
from tencentcloud.autoscaling.v20180419 import autoscaling_client

def get_as_client(region: str) -> autoscaling_client.AutoscalingClient:
    """获取 AS 客户端
    
    Args:
        region: 地域
        
    Returns:
        AutoscalingClient: AS 客户端实例
    """
    # 从环境变量获取密钥
    secret_id = os.getenv("TENCENTCLOUD_SECRET_ID")
    secret_key = os.getenv("TENCENTCLOUD_SECRET_KEY")
    
    if not secret_id or not secret_key:
        raise ValueError("请设置环境变量: TENCENTCLOUD_SECRET_ID 和 TENCENTCLOUD_SECRET_KEY")
    
    # 实例化一个认证对象
    cred = credential.Credential(secret_id, secret_key)
    
    # 实例化一个http选项
    httpProfile = HttpProfile()
    httpProfile.endpoint = "as.tencentcloudapi.com"
    
    # 实例化一个client选项
    clientProfile = ClientProfile()
    clientProfile.httpProfile = httpProfile
    clientProfile.request_client = "MCP-Server"
    
    # 实例化要请求产品的client对象
    return autoscaling_client.AutoscalingClient(cred, region, clientProfile)



"""
腾讯云 AS 相关操作工具模块
"""
import json
from tencentcloud.autoscaling.v20180419 import models
from .client import get_as_client

def create_auto_scaling_group(region: str, params: dict) -> str:
    """创建伸缩组
    
    Args:
        region: 地域
        params: 创建参数，包含：
            - AutoScalingGroupName: 伸缩组名称
            - LaunchConfigurationId: 启动配置ID
            - MaxSize: 最大实例数
            - MinSize: 最小实例数
            - VpcId: VPC ID
            - SubnetIds: 子网ID列表
            - DesiredCapacity: 期望实例数
            
    Returns:
        str: API响应结果的JSON字符串
    """
    client = get_as_client(region)
    req = models.CreateAutoScalingGroupRequest()
    req.from_json_string(json.dumps(params))
    resp = client.CreateAutoScalingGroup(req)
    return resp.to_json_string()

def describe_auto_scaling_groups(region: str, auto_scaling_group_ids: list[str] = None,
                               filters: list[dict] = None, limit: int = None,
                               offset: int = None) -> str:
    """查询伸缩组
    
    Args:
        region: 地域
        auto_scaling_group_ids: 伸缩组ID列表
        filters: 过滤条件
        limit: 返回数量
        offset: 偏移量
        
    Returns:
        str: API响应结果的JSON字符串
    """
    client = get_as_client(region)
    req = models.DescribeAutoScalingGroupsRequest()
    
    params = {}
    if auto_scaling_group_ids:
        params["AutoScalingGroupIds"] = auto_scaling_group_ids
    if filters:
        params["Filters"] = filters
    if limit is not None:
        params["Limit"] = limit
    if offset is not None:
        params["Offset"] = offset
        
    if params:
        req.from_json_string(json.dumps(params))
    resp = client.DescribeAutoScalingGroups(req)
    return resp.to_json_string()

def modify_auto_scaling_group(region: str, auto_scaling_group_id: str,
                            params: dict) -> str:
    """修改伸缩组
    
    Args:
        region: 地域
        auto_scaling_group_id: 伸缩组ID
        params: 修改参数
        
    Returns:
        str: API响应结果的JSON字符串
    """
    client = get_as_client(region)
    req = models.ModifyAutoScalingGroupRequest()
    
    params["AutoScalingGroupId"] = auto_scaling_group_id
    req.from_json_string(json.dumps(params))
    resp = client.ModifyAutoScalingGroup(req)
    return resp.to_json_string()

def enable_auto_scaling_group(region: str, auto_scaling_group_id: str) -> str:
    """启用伸缩组
    
    Args:
        region: 地域
        auto_scaling_group_id: 伸缩组ID
        
    Returns:
        str: API响应结果的JSON字符串
    """
    client = get_as_client(region)
    req = models.EnableAutoScalingGroupRequest()
    
    params = {
        "AutoScalingGroupId": auto_scaling_group_id
    }
    req.from_json_string(json.dumps(params))
    resp = client.EnableAutoScalingGroup(req)
    return resp.to_json_string()

def disable_auto_scaling_group(region: str, auto_scaling_group_id: str) -> str:
    """停用伸缩组
    
    Args:
        region: 地域
        auto_scaling_group_id: 伸缩组ID
        
    Returns:
        str: API响应结果的JSON字符串
    """
    client = get_as_client(region)
    req = models.DisableAutoScalingGroupRequest()
    
    params = {
        "AutoScalingGroupId": auto_scaling_group_id
    }
    req.from_json_string(json.dumps(params))
    resp = client.DisableAutoScalingGroup(req)
    return resp.to_json_string()

def execute_scaling_policy(region: str, auto_scaling_group_id: str,
                         operation: str, adjustment_type: str = None,
                         adjustment_value: int = None) -> str:
    """执行伸缩策略
    
    Args:
        region: 地域
        auto_scaling_group_id: 伸缩组ID
        operation: 操作类型，取值范围：SCALE_OUT（扩容），SCALE_IN（缩容）
        adjustment_type: 调整类型，取值范围：CHANGE_IN_CAPACITY, EXACT_CAPACITY, PERCENT_CHANGE_IN_CAPACITY
        adjustment_value: 调整值，正数表示增加实例，负数表示减少实例
        
    Returns:
        str: API响应结果的JSON字符串
    """
    client = get_as_client(region)
    req = models.ExecuteScalingPolicyRequest()
    
    params = {
        "AutoScalingGroupId": auto_scaling_group_id,
        "Operation": operation
    }
    if adjustment_type:
        params["AdjustmentType"] = adjustment_type
    if adjustment_value is not None:
        params["AdjustmentValue"] = adjustment_value
        
    req.from_json_string(json.dumps(params))
    resp = client.ExecuteScalingPolicy(req)
    return resp.to_json_string()

def modify_desired_capacity(region: str, auto_scaling_group_id: str,
                          desired_capacity: int) -> str:
    """修改期望实例数
    
    Args:
        region: 地域
        auto_scaling_group_id: 伸缩组ID
        desired_capacity: 期望实例数
        
    Returns:
        str: API响应结果的JSON字符串
    """
    client = get_as_client(region)
    req = models.ModifyDesiredCapacityRequest()
    
    params = {
        "AutoScalingGroupId": auto_scaling_group_id,
        "DesiredCapacity": desired_capacity
    }
    req.from_json_string(json.dumps(params))
    resp = client.ModifyDesiredCapacity(req)
    return resp.to_json_string()
